!include "MUI.nsh"
!define MUI_ICON "applications\eden\static\favicon.ico"
!include "FileFunc.nsh"
!include "LogicLib.nsh"

Name "Sahana Eden"
OutFile "sahana-eden-setup.exe"
RequestExecutionLevel admin
InstallDir C:\Users\Public\SahanaEden
!define INSTALLATIONNAME "SahanaEden"

!insertmacro MUI_PAGE_WELCOME
Page custom getSetupMode
!define MUI_PAGE_CUSTOMFUNCTION_SHOW dir_pre ; Function for the "no USB Drive found" message
!insertmacro MUI_PAGE_DIRECTORY
Page custom getPrefs
!insertmacro MUI_PAGE_INSTFILES
Page custom sahanaInfo
!insertmacro MUI_UNPAGE_CONFIRM
!insertmacro MUI_UNPAGE_INSTFILES
!insertmacro MUI_LANGUAGE "English"



Var /GLOBAL IP
Var /GLOBAL Port
Var /GLOBAL Password
Var /GLOBAL Portable
Var /GLOBAL tmpvar


LangString PAGE_WEB2PY_TITLE ${LANG_ENGLISH} "Web2Py Configuration"
LangString PAGE_WEB2PY_SUBTITLE ${LANG_ENGLISH} "Please enter the password, IP and port Web2Py should use."

LangString PAGE_MODE_TITLE ${LANG_ENGLISH} "Installation method"
LangString PAGE_MODE_SUBTITLE ${LANG_ENGLISH} "Please specify whether you want to use the Portable mode or not."

LangString PAGE_SAHANAINFO_TITLE ${LANG_ENGLISH} "Installation complete"
LangString PAGE_SAHANAINFO_SUBTITLE ${LANG_ENGLISH} "Please read this information concerning Sahana Eden."


Function ".onInit"
  InitPluginsDir
  File /oname=$PLUGINSDIR\setup-cust.ini "setup-cust.ini"
  File /oname=$PLUGINSDIR\setup-mode.ini "setup-mode.ini"
  File /oname=$PLUGINSDIR\setup-sahanainfo.ini "setup-sahanainfo.ini"
FunctionEnd


Function getSetupMode ; Page to choose wether local or protable installation
  !insertmacro MUI_HEADER_TEXT $(PAGE_MODE_TITLE) $(PAGE_MODE_SUBTITLE)
  Push $R0 ; Dont care about the stack
  InstallOptions::dialog $PLUGINSDIR\setup-mode.ini
  Pop $R0 ; Dont care about the stack
  ReadINIStr $Portable "$PLUGINSDIR\setup-mode.ini" "Field 4" "state"
  Pop $R0 ; Dont care about the stack
  ${If} $Portable == ""
        StrCpy $Portable "0"
  ${EndIf}
  ${If} $Portable == "1"
        ${GetDrives} "FDD" "setUSBDir" ; USB Drives are recognized as FDD
  ${Else}
         StrCpy $INSTDIR "C:\Users\Public\SahanaEden"
  ${EndIf}

FunctionEnd

Function sahanaInfo ; Eden info page
         !insertmacro MUI_HEADER_TEXT $(PAGE_SAHANAINFO_TITLE) $(PAGE_SAHANAINFO_SUBTITLE)
         InstallOptions::dialog $PLUGINSDIR\setup-sahanainfo.ini
FunctionEnd

Function setUSBDir
         ${DriveSpace} "$9" "/S=M" $tmpvar ; reads the remaining space of Drive $9 into $tempvar
         ${If} $tmpvar < 90 ; Drive either not connected or too small
               Push $0
         ${Else}
               Push "StopGetDrives"
               StrCpy $INSTDIR "$9SahanaEden"
         ${EndIf}
FunctionEnd
  
Function getPrefs ; Web2py config page
  !insertmacro MUI_HEADER_TEXT $(PAGE_WEB2PY_TITLE) $(PAGE_WEB2PY_SUBTITLE)
  Push $R0 ; Dont care about the stack
  InstallOptions::dialog $PLUGINSDIR\setup-cust.ini
  Pop $R0 ; Dont care about the stack
  ReadINIStr $Password "$PLUGINSDIR\setup-cust.ini" "Field 2" "state"
  ReadINIStr $IP "$PLUGINSDIR\setup-cust.ini" "Field 3" "state"
  ReadINIStr $Port "$PLUGINSDIR\setup-cust.ini" "Field 4" "state"
  Pop $R0 ; Dont care about the stack
  ${If} $Password == ""
        StrCpy $Password "password"
  ${EndIf}
  ${If} $IP == ""
        StrCpy $IP "127.0.0.1"
  ${EndIf}
  ${If} $Port == ""
        StrCpy $Port "8000"
  ${EndIf}
FunctionEnd

Function GetLastPart
         Exch $R0 ; input
         Exch
         Exch $R1 ; divider str
         Push $R2
         Push $R3
         Push $R4
         Push $R5
         StrCpy $R2 -1
         StrLen $R4 $R0
         StrLen $R5 $R1
         Loop:
              IntOp $R2 $R2 + 1
              StrCpy $R3 $R0 $R5 $R2
              StrCmp $R3 $R1 Chop
              StrCmp $R2 $R4 0 Loop
              StrCpy $R0 ""
              StrCpy $R1 ""
              Goto Done
        Chop:
              StrCpy $R1 $R0 $R2
              IntOp $R2 $R2 + $R5
              StrCpy $R0 $R0 "" $R2
        Done:
             Pop $R5
             Pop $R4
             Pop $R3
             Pop $R2
             Exch $R1 ; before
             Exch
             Exch $R0 ; after
FunctionEnd

Function dir_pre ; Shows a message if portable mode and no device found or device was too small
         ${If} $Portable == 1
               ${If} $INSTDIR == "C:\Users\Public\SahanaEden"
                MessageBox MB_OK|MB_TOPMOST|MB_ICONEXCLAMATION "No removable device (> 90MB) found.$\r$\nPlease select it manually."
               ${EndIf}
         ${EndIf}
FunctionEnd

Section ""
  SetOutPath $INSTDIR
  ; Files start
   File /r /x "end-user-install.nsi" "setup-cust.ini" "setup-mode.ini" "setup-sahanainfo.ini" "*"
  ; Files end

  ${If} $Portable == "0"
        ; We are not in the portable-installation mode
        WriteUninstaller $INSTDIR\uninstall.exe
        ; Register eden in Windows' softwaremanagemnt
        WriteRegStr HKLM "Software\Microsoft\Windows\CurrentVersion\Uninstall\${INSTALLATIONNAME}" "DisplayName" "Sahana Eden"
        WriteRegStr HKLM "Software\Microsoft\Windows\CurrentVersion\Uninstall\${INSTALLATIONNAME}" "UninstallString" '"$INSTDIR\uninstall.exe"'
        WriteRegStr HKLM "Software\Microsoft\Windows\CurrentVersion\Uninstall\${INSTALLATIONNAME}" "Publisher" "Sahana Software Foundation"
        WriteRegDWORD HKLM "Software\Microsoft\Windows\CurrentVersion\Uninstall\${INSTALLATIONNAME}" "NoModify" 1
        WriteRegDWORD HKLM "Software\Microsoft\Windows\CurrentVersion\Uninstall\${INSTALLATIONNAME}" "NoRepair" 1
        ;create desktop shortcut
        SetShellVarContext all ; Windows Vista/7 Workaround for the strange UAC
        CreateShortCut "$DESKTOP\Sahana Eden.lnk" "$INSTDIR\startweb2py.bat" "" "$INSTDIR\applications\eden\static\favicon.ico" 0
        ;create start-menu shortcut
        CreateShortCut "$SMPROGRAMS\Sahana Eden.lnk" "$INSTDIR\startweb2py.bat" "" "$INSTDIR\applications\eden\static\favicon.ico" 0
  ${Else}
        ; We are in portable mode
        ; Create the autorun.ini
        Push ":\" ; divider str
        Push "$instdir" ; input string (full path in this case)
        Call GetLastPart
        Pop $R1 ; last part (relative path)
        Pop $R0 ; first part (Drive letter)
        FileOpen $4 "$R0:\autorun.inf" w
        FileWrite $4 "[AutoRun]$\r$\nopen=$R1\startweb2py.bat$\r$\nicon=$R1\applications\eden\static\favicon.ico$\r$\naction=Launch Sahana Eden$\r$\nLabel=SahanaEden"
        FileClose $4
        FileOpen $4 "$R0:\start-eden.bat" w
        FileWrite $4 "@echo off$\r$\ncd $R1$\r$\nstart startweb2py.bat$\r$\nexit"
        FileClose $4
  ${EndIf}
  FileOpen $4 "$INSTDIR\startweb2py.bat" w
  FileWrite $4 "@echo off$\r$\nstart web2py.exe  -i $IP -p $Port -a $Password$\r$\necho Please Wait...$\r$\nping -n 3 127.0.0.1 > NUL$\r$\nstart http://127.0.0.1:$Port$\r$\nexit"
  FileClose $4
SectionEnd

Section "Uninstall"
  RMDir /r /REBOOTOK $INSTDIR
  DeleteRegKey HKLM "Software\Microsoft\Windows\CurrentVersion\Uninstall\${INSTALLATIONNAME}"
  SetShellVarContext all ; Windows Vista/7 Workaround for the strange UAC
  Delete "$DESKTOP\Sahana Eden.lnk"
  Delete "$SMPROGRAMS\Sahana Eden.lnk"
SectionEnd